{*************************************************************}
{            File Storage Component for Delphi 16/32          }
{ Version:   1.0                                              }
{ Author:    Aleksey Kuznetsov, Kiev, Ukraine                 }
{              (Xacker), ,          }
{ E-Mail:    xacker@phreaker.net                              }
{ Home Page: http://www.angen.net/~xacker/                    }
{ Created:   March, 22, 1999                                  }
{ Modified:  March, 22, 1999                                  }
{ Legal:     Copyright (c) 1999 by Aleksey Xacker             }
{*************************************************************}
{   TFileStorage (English):                                   }
{ Given component is intended for storing data (any, taken    }
{ from a file specified by the developer) in a body of .exe   }
{ file. Access to a file (being in a body of .exe project),   }
{ realized through the "Data: Pointer".                       }
{*************************************************************}
{   TFileStorage (Russian):                                   }
{       (  }
{     )   .exe .}
{    (   .exe )             }
{    Data,  Pointer.          }
{*************************************************************}
{  PROPERTIES:                                                }
{   property IsEmpty: Boolean;                                }
{   property GetData: Pointer;                                }
{   property DataSize: LongInt;                               }
{*************************************************************}
{                     IMPORTANT NOTE:                         }
{ This software is provided 'as-is', without any express or   }
{ implied warranty. In no event will the author be held       }
{ liable for any damages arising from the use of this         }
{ software.                                                   }
{ Permission is granted to anyone to use this software for    }
{ any purpose, including commercial applications, and to      }
{ alter it and redistribute it freely, subject to the         }
{ following restrictions:                                     }
{ 1. The origin of this software must not be misrepresented,  }
{    you must not claim that you wrote the original software. }
{    If you use this software in a product, an acknowledgment }
{    in the product documentation would be appreciated but is }
{    not required.                                            }
{ 2. Altered source versions must be plainly marked as such,  }
{    and must not be misrepresented as being the original     }
{    software.                                                }
{ 3. This notice may not be removed or altered from any       }
{    source distribution.                                     }
{*************************************************************}
unit FStorage;

interface

uses
  {$IFDEF WIN32} Windows, {$ELSE} WinTypes, WinProcs,{$ENDIF}
  Classes, Controls, Dialogs, DsgnIntf;

type
  TFileStorage = class(TComponent)
  private
    FDataSize: LongInt;
    Data: Pointer;
    Hand: THandle;
    FEmpty: Boolean;
    FStr: TFileStream;
    FFileName: String;
  protected
    procedure ReadState(Reader: TReader); override;
    procedure WriteState(Writer: TWriter); override;
    procedure StoreFile(vFileName: String); virtual;
    procedure SetFileName(Value: String); { obsolete }
    procedure SetDataSize(Value: LongInt); { obsolete }
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    property IsEmpty: Boolean read FEmpty;
    property GetData: Pointer read Data;
    function ExtractFile(TargetFileName: String): Boolean;
  published
    property FileName: String read FFileName write SetFileName;
    property DataSize: LongInt read FDataSize write SetDataSize;
  end;

 TFileStorageCompEditor = class(TComponentEditor)
    OpenDialog: TOpenDialog;
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerbCount: Integer; override;
    function GetVerb(Index: Integer): string; override;
    constructor Create(aOwner: TComponent; aDesigner: TFormDesigner); override;
    destructor Destroy; override;
  end;

procedure Register;

implementation

procedure TFileStorage.StoreFile(vFileName: String);
var
  f: File;
  Block: Word;
begin
  if not FEmpty then
   begin
    GlobalUnlock(Hand);
    GlobalFree(Hand);
   end;
  AssignFile(f, vFileName);
  Reset(f, 1);
  FDataSize := FileSize(f);
  CloseFile(f);
  Hand := GlobalAlloc(gmem_Moveable or gmem_ZeroInit, FDataSize);
  Data := GlobalLock(Hand);
  try
   FStr := TFileStream.Create(vFileName, 0);
   FStr.Read(Data^, FDataSize);
   FStr.Free;
   FEmpty := False;
   FFileName := vFileName;
  except
   MessageDlg('Read error or disk is corrupted.', mtError, [mbOk], 0);
   GlobalUnlock(Hand);
   GlobalFree(Hand);
   FDataSize := 0;
   FFileName := '';
  end;
end;

constructor TFileStorage.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  FEmpty := True;
end;

destructor TFileStorage.Destroy;
begin
  if not FEmpty then
   begin
    GlobalUnlock(Hand);
    GlobalFree(Hand);
   end;
  inherited Destroy;
end;

procedure TFileStorage.ReadState(Reader: TReader);
begin
  inherited ReadState(Reader);
  FEmpty := Reader.ReadBoolean;
  if not FEmpty then
   begin
    FDataSize := Reader.ReadInteger;
    Hand := GlobalAlloc(gmem_Moveable or gmem_ZeroInit, FDataSize);
    Data := GlobalLock(Hand);
    Reader.Read(Data^, FDataSize);
   end;
end;

procedure TFileStorage.WriteState(Writer: TWriter);
begin
  inherited WriteState(Writer);
  Writer.WriteBoolean(FEmpty);
  if not FEmpty then
   begin
    Writer.WriteInteger(FDataSize);
    Writer.Write(Data^, FDataSize);
   end;
end;

function TFileStorage.ExtractFile(TargetFileName: String): Boolean;
begin
  try
    FStr := TFileStream.Create(TargetFileName, fmCreate);
    FStr.Write(Data^, FDataSize);
    FStr.Free;
    Result := True;
  except
    Result := False;
  end;
end;

procedure TFileStorage.SetFileName(Value: String); { obsolete }
begin
end;

procedure TFileStorage.SetDataSize(Value: LongInt); { obsolete }
begin
end;

constructor TFileStorageCompEditor.Create(aOwner: TComponent; aDesigner: TFormDesigner);
begin
  inherited Create(aOwner, aDesigner);
  OpenDialog := TOpenDialog.Create(nil);
end;

destructor TFileStorageCompEditor.Destroy;
begin
  OpenDialog.Free;
  inherited Destroy;
end;

procedure TFileStorageCompEditor.ExecuteVerb(Index: Integer);
begin
  if Index + 1 = GetVerbCount then
  else inherited ExecuteVerb(Index);

  OpenDialog.Filter := 'All files (*.*)|*.*';
  OpenDialog.Options := [ofFileMustExist];
  if OpenDialog.Execute then
   (Component as TFileStorage).StoreFile(OpenDialog.FileName);
end;

function TFileStorageCompEditor.GetVerbCount: Integer;
begin
  Result := inherited GetVerbCount + 1;
end;

function TFileStorageCompEditor.GetVerb(Index: Integer): string;
begin
  if Index + 1 = GetVerbCount then Result:='&Load Data!...'
  else Result := inherited GetVerb(Index);
end;

procedure Register;
begin
  RegisterComponents('Xacker', [TFileStorage]);
  RegisterComponentEditor(TFileStorage, TFileStorageCompEditor);
end;

end.
